/*
 * Copyright © 2009 Perseus Project - Tufts University <http://www.perseus.tufts.edu>
 *
 * This file is part of UniCollatorPerseus.
 *
 * AlignmentPerseus is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * AlignmentPerseus is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with AlignmentPerseus.  If not, see <http://www.gnu.org/licenses/>.
 */

package eu.himeros.alignment;

import eu.himeros.util.IoManager;
import java.io.BufferedWriter;
import java.util.Vector;

/**
 * Perform multiple alignment.
 *
 * @author Federico Boschetti <federico.boschetti.73@gmail.com>
 */
public class MultiAligner {
    private Vector<String> errorPatternFileNames=new Vector<String>();
    private StringBuffer[] sbs=null;
    private BufferedWriter bw=null;
    /**
     * Default Constructor.
     */
    public MultiAligner(){}

    /**
     *
     * @param fileName
     */
    public void addErrorPatternFile(String fileName){
        errorPatternFileNames.add(fileName);
    }

    /**
     * Execute the multiple alignment.
     *
     * @param v the vector with text to be aligned.
     * @param multiAlignedFileName the alignment report file name.
     * @param mergedFileName the aligned text output file name.
     * @param upperTransFileName the file to transcode lowercase in uppercase characters.
     */
    //TODO create methods to set gap characters and filters.

    public void exec(Vector<String>[] v, String mergedFileName){
        exec(v,null,mergedFileName);
    }

    public void exec(Vector<String>[] v, String multiAlignedFileName, String mergedFileName){
        int n=3;
        if(multiAlignedFileName!=null){
            bw=IoManager.getWriter(multiAlignedFileName);
        }
        BufferedWriter bwText=IoManager.getWriter(mergedFileName); // aligned text
        StringAligner alg=new StringAligner(new UpperCaseSimEvaluator());
        MutualCorrector mcorr=new MutualCorrector(errorPatternFileNames);
        AlignedSpellChecker asc=new AlignedSpellChecker();
        sbs=new StringBuffer[n+1];
        alg.setGapPenalty(-1);
        String[][] algs=new String[n][];
        int len=v[0].size();
        for(int i=0;i<len;i++){
            for(int j=0;j<n+1;j++){
                sbs[j]=new StringBuffer(5000);
            }
            alg.setGapChar("#".charAt(0));
            algs[0]=alg.align(v[0].get(i),v[1].get(i));
            alg.setGapChar("\u00B0".charAt(0));
            algs[1]=alg.align(v[2].get(i), v[0].get(i)); // v[3] instead of v[0]
            alg.setGapChar("@".charAt(0));
            algs[2]=alg.align(algs[0][1],algs[1][0]);
            algs[0][0]=alg.adjustGap(algs[0][0],algs[2][0]);
            algs[1][1]=alg.adjustGap(algs[1][1],algs[2][1]);
            algs[0][0]=algs[0][0].replaceAll("[#@]","\u00B0");
            algs[2][0]=algs[2][0].replaceAll("[#@]","\u00B0");
            algs[2][1]=algs[2][1].replaceAll("[#@]","\u00B0");
            Vector<String> strV=new Vector<String>();
            strV.add(algs[0][0]);
            strV.add(algs[2][0]);
            strV.add(algs[2][1]);
            String mergedStr=mcorr.merge(strV);
            String correctedStr=asc.correct(strV, mergedStr);
            correctedStr=correctedStr.replaceAll("^","");
            try{
                sbs[0].append(algs[0][0]);
                sbs[1].append(algs[2][0]);
                sbs[2].append(algs[2][1]);
                sbs[3].append(mergedStr);
            correctedStr=correctedStr.replaceAll("[\u00B0^]", "").trim();
            correctedStr=correctedStr.replaceAll("\u00B6","\n");
            correctedStr=correctedStr.replaceAll(" +"," ")+" ";
            correctedStr=correctedStr.replaceAll("([\u0370-\u03FF\u1F00-\u1FFF])([^\u0370-\u03FF\u1F00-\u1FFF \n])([\u0370-\u03FF\u1F00-\u1FFF])","$1$3");
            bwText.write(correctedStr);
            }catch(Exception ex){ex.printStackTrace();}
        }
        try{
            if(multiAlignedFileName!=null){
                writeReport();
                bw.close();
            }
            bwText.close();
        }catch(Exception ex){ex.printStackTrace();}
    }
    
    private void writeReport(){
        int mod=80;
        int len=sbs[0].length();
        int offset=0;
        int beg=0;
        int end=0;
        try{
            while(offset<len){
                beg=offset;
                end=beg+mod;
                if(end>len) end=len;
                bw.write(sbs[0].substring(beg,end));bw.newLine();
                bw.write(sbs[1].substring(beg,end));bw.newLine();
                bw.write(sbs[2].substring(beg,end));bw.newLine();
                //bw.write(sbs[3].substring(beg,end));bw.newLine();
                bw.newLine();
                offset+=mod;
            }
        }catch(Exception ex){ex.printStackTrace();}
    }


}